from sys import platform
from datetime import datetime as dt

import praw
from slacker import Slacker

from recho import __version__ as recho_version


def build_user_agent():
    recho_version_trunk = recho_version.split("+")[0]
    user_agent = '{0}:recho:{1} (by /u/IHKAS1984)'
    user_agent = user_agent.format(platform, recho_version_trunk)

    return user_agent


def get_reddit_posts_since(redditor_name, timestamp):
    """ Yields all reddit posts since timestamp for redditor
    """
    reddit = praw.Reddit(user_agent=build_user_agent())
    new_comments = list()
    for comment in reddit.get_redditor(redditor_name).get_comments('week'):
        if timestamp > dt.fromtimestamp(comment.created):
            break

        new_comments.append(comment)

    return sorted(new_comments, key=lambda x: x.created)


def post_to_slack(slack_config, comments):
    """ Post the given comments to the configured slack channel
    """
    slack = Slacker(slack_config['token'])

    for comment in comments:
        message = format_for_slack(comment)
        slack.chat.post_message(slack_config['channel'], message, as_user=True)


def format_for_slack(comment):
    """ Format a reddit comment for slack
    """
    slack_post = ("*User*: {user_name}\n"
                  "*Thread*: {thread}\n"
                  "*Permalink*: `{permalink}`\n"
                  "```"
                  "{body}"
                  "```")
    user_name = comment.author
    thread = comment.link_title[:75]
    permalink = comment.permalink
    body = comment.body

    return slack_post.format(**locals())



import os
import argparse
from datetime import timedelta, datetime as dt
from configparser import ConfigParser

# import appdirs

from recho.recho import get_reddit_posts_since, post_to_slack

CONFIG_NAME = '.recho.ini'

TS_NAME = '.recho_timestamp'
TS_FILEPATH = os.path.join(os.path.dirname(__file__), TS_NAME)
EST = timedelta(seconds=7200)


class RechoError(Exception):
    pass


def get_args():
    parser = argparse.ArgumentParser()
    parser.add_argument(
        'redditor', type=str,
        help="The name of the target Redditor"
    )

    return parser.parse_args()


def load_config():
    config = ConfigParser()
    config_path = os.path.join(os.path.expanduser('~'), CONFIG_NAME)

    if not os.path.exists(config_path):
        raise RechoError("Config file must be present in ~/.recho.ini")

    config.read(config_path)

    return config


def get_last_seen():
    """ Return the last time a check was done
    """
    if not os.path.exists(TS_FILEPATH):
        last_seen = dt.utcnow() + EST
    else:
        with open(TS_FILEPATH, 'r') as r:
            last_seen = dt.fromtimestamp(float(r.read()))

    return last_seen


def main():
    # Load CLI args
    args = get_args()

    # Load config
    config = load_config()

    # Get last timestamp
    last_seen = get_last_seen()

    # Get comments from reddit
    comments = get_reddit_posts_since(args.redditor, last_seen)

    if comments:
        # Post posts to slack
        post_to_slack(config['slack'], comments)

    # Write new timestamp
    new_ts = (dt.now() + EST).timestamp()
    with open(TS_FILEPATH, 'w') as w:
        w.write(str(new_ts))



"""
Use setup tools to setup recho as a standard python package
"""
from os import path
from setuptools import find_packages, setup

import versioneer

HERE = path.dirname(path.abspath(__file__))

# Get the long description from the README file
with open(path.join(HERE, 'README.rst')) as f:
    LONG_DESCRIPTION = f.read()

INSTALL_REQUIRES = [
    'retry',
    'appdirs==1.4.0',
    'configparser==3.5.0',
    'praw==3.6.0',
    'slacker==0.9.30',
]

setup(
    name="recho",
    version=versioneer.get_version(),
    cmdclass=versioneer.get_cmdclass(),
    author='Levi Noecker',
    author_email='levi [at] dank [dot] city',
    url='https://github.com/DankCity/recho',
    keywords=['reddit', 'slack', 'bot', 'slack bot', 'dankbot', 'recho'],
    description="Post the latest Reddit comments from a specific User to a specific Slack channel",
    packages=find_packages(),
    license='MIT',
    long_description=LONG_DESCRIPTION,
    test_suite="tests",
    tests_require=['tox'],
    install_requires=INSTALL_REQUIRES,
    entry_points={
        "console_scripts": [
            "recho=recho.cli:main",
        ]
    },
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Topic :: Communications :: Chat',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy',
    ],
)
